/*---------------------------------------------------------------------------\
| Project:  PIC18F458 CAN Node Core Firmware
|
| File:     config.h
|
| Descr.:   THIS IS AN APPLICATION SPECIFIC FILE. MODIFY IT, FOR GODS SAKE.
|           #defines to set the various interrupts and handlers correctly.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 08/12/2002 | Created after playing with function pointers.
\---------------------------------------------------------------------------*/

#ifndef _CONFIG_H    // Don't include more than once (last line = #endif)
#define _CONFIG_H


/*---------------------------------------------------------------------------\
|
| Choose the appropriate setting for each possible interrupt in the PIC:
|
| 1. Choose whether you want the core interrupt handlers to be used at
|    all. For example, if you NEVER use the UART (serial port), then you
|    should turn off the interrupt service routine call so that there's less
|    junk to look for in the main ISR. NB: If you DO enable serial interrupts
|    after you have turned off the serial ISR, then the  PIC will go into
|    loop-forever mode as it continuously interrupts since the interrupt will
|    never be cleared. So don't do that.
|
|       INTS_CORE_ISR_ENABLED
|       INTS_CORE_ISR_DISABLED
|
| 1. Now choose interrupt priorities. You can choose from two levels:
|
|   INTS_LOW_PRIORITY_INTERRUPT
|   INTS_HIGH_PRIORITY_INTERRUPT
|
| ...although note the exception that INT0 may only be HIGH.
|
| 2. Choose whether to enable the application specific interrupt handlers.
|
| If we were cool then the core firmware would be an OS and we'd have
| libraries and system calls and callbacks (function pointers) to
| application specific interrupt handlers. Screw that. We're working on a
| skeleton core with an 8bit PIC. So, we pre-define the handler names
| and only include the prototype in the .h files the calls in the ints.c if
| they're enabled. Puke. Your choices:
|
|   INTS_HOOK_ENABLED
|   INTS_HOOK_DISABLED
|
\---------------------------------------------------------------------------*/

// Timer interrupts

#define INTS_TIMER3_ISR_USED            INTS_CORE_ISR_DISABLED
#define INTS_TIMER3_INT_PRIORITY        INTS_LOW_PRIORITY_INTERRUPT
#define INTS_TIMER3_APPLICATION_HOOK    INTS_HOOK_DISABLED

#define INTS_TIMER2_ISR_USED            INTS_CORE_ISR_ENABLED
#define INTS_TIMER2_INT_PRIORITY        INTS_HIGH_PRIORITY_INTERRUPT
#define INTS_TIMER2_APPLICATION_HOOK    INTS_HOOK_DISABLED

#define INTS_TIMER1_ISR_USED            INTS_CORE_ISR_DISABLED
#define INTS_TIMER1_INT_PRIORITY        INTS_LOW_PRIORITY_INTERRUPT
#define INTS_TIMER1_APPLICATION_HOOK    INTS_HOOK_DISABLED

#define INTS_TIMER0_ISR_USED            INTS_CORE_ISR_DISABLED
#define INTS_TIMER0_INT_PRIORITY        INTS_LOW_PRIORITY_INTERRUPT
#define INTS_TIMER0_APPLICATION_HOOK    INTS_HOOK_DISABLED

// Capture/Compare/PWM interrupts

#define INTS_CCP_ISR_USED               INTS_CORE_ISR_DISABLED
#define INTS_CCP_INT_PRIORITY           INTS_LOW_PRIORITY_INTERRUPT
#define INTS_CCP_APPLICATION_HOOK       INTS_HOOK_DISABLED

#define INTS_ECCP_ISR_USED              INTS_CORE_ISR_DISABLED
#define INTS_ECCP_INT_PRIORITY          INTS_LOW_PRIORITY_INTERRUPT
#define INTS_ECCP_APPLICATION_HOOK      INTS_HOOK_DISABLED

// Low Voltage (brownout) and Comparator Interrupt

#define INTS_LVD_ISR_USED               INTS_CORE_ISR_DISABLED
#define INTS_LVD_INT_PRIORITY           INTS_LOW_PRIORITY_INTERRUPT
#define INTS_LVD_APPLICATION_HOOK       INTS_HOOK_DISABLED

#define INTS_CMP_ISR_USED               INTS_CORE_ISR_DISABLED
#define INTS_CMP_INT_PRIORITY           INTS_LOW_PRIORITY_INTERRUPT
#define INTS_CMP_APPLICATION_HOOK       INTS_HOOK_DISABLED

// Port interrupts: Int0 (RB0), Int1 (RB1), Port B interrupt on change

#define INTS_INT0_ISR_USED              INTS_CORE_ISR_ENABLED
        // INT0 interrupts may only be high priority.
#define INTS_INT0_APPLICATION_HOOK      INTS_HOOK_ENABLED

#define INTS_INT1_ISR_USED              INTS_CORE_ISR_DISABLED
#define INTS_INT1_INT_PRIORITY          INTS_LOW_PRIORITY_INTERRUPT
#define INTS_INT1_APPLICATION_HOOK      INTS_HOOK_DISABLED

#define INTS_PBIOC_ISR_USED             INTS_CORE_ISR_DISABLED
#define INTS_PBIOC_INT_PRIORITY         INTS_LOW_PRIORITY_INTERRUPT
#define INTS_PBIOC_APPLICATION_HOOK     INTS_HOOK_DISABLED

// ADC interrupts

#define INTS_ADC_ISR_USED               INTS_CORE_ISR_DISABLED
#define INTS_ADC_INT_PRIORITY           INTS_LOW_PRIORITY_INTERRUPT
#define INTS_ADC_APPLICATION_HOOK       INTS_HOOK_DISABLED


// USART interrupts

#define INTS_SERIAL_TX_ISR_USED         INTS_CORE_ISR_ENABLED
#define INTS_SERIAL_TX_INT_PRIORITY     INTS_LOW_PRIORITY_INTERRUPT
#define INTS_SERIAL_TX_APPLICATION_HOOK INTS_HOOK_DISABLED

#define INTS_SERIAL_RX_ISR_USED         INTS_CORE_ISR_ENABLED
#define INTS_SERIAL_RX_INT_PRIORITY     INTS_LOW_PRIORITY_INTERRUPT
#define INTS_SERIAL_RX_APPLICATION_HOOK INTS_HOOK_DISABLED

#define INTS_SSP_ISR_USED               INTS_CORE_ISR_DISABLED
#define INTS_SSP_INT_PRIORITY           INTS_LOW_PRIORITY_INTERRUPT
#define INTS_SSP_APPLICATION_HOOK       INTS_HOOK_DISABLED

// CAN interrupts

#define INTS_CAN_TX2_ISR_USED           INTS_CORE_ISR_ENABLED
#define INTS_CAN_TX2_INT_PRIORITY       INTS_LOW_PRIORITY_INTERRUPT
#define INTS_CAN_TX2_APPLICATION_HOOK   INTS_HOOK_DISABLED

#define INTS_CAN_TX1_ISR_USED           INTS_CORE_ISR_ENABLED
#define INTS_CAN_TX1_INT_PRIORITY       INTS_LOW_PRIORITY_INTERRUPT
#define INTS_CAN_TX1_APPLICATION_HOOK   INTS_HOOK_DISABLED

#define INTS_CAN_TX0_ISR_USED           INTS_CORE_ISR_ENABLED
#define INTS_CAN_TX0_INT_PRIORITY       INTS_LOW_PRIORITY_INTERRUPT
#define INTS_CAN_TX0_APPLICATION_HOOK   INTS_HOOK_DISABLED

#define INTS_CAN_RX0_ISR_USED           INTS_CORE_ISR_ENABLED
#define INTS_CAN_RX0_INT_PRIORITY       INTS_LOW_PRIORITY_INTERRUPT
#define INTS_CAN_RX0_APPLICATION_HOOK   INTS_HOOK_DISABLED

#define INTS_CAN_ERR_ISR_USED           INTS_CORE_ISR_DISABLED
#define INTS_CAN_ERR_INT_PRIORITY       INTS_LOW_PRIORITY_INTERRUPT
#define INTS_CAN_ERR_APPLICATION_HOOK   INTS_HOOK_DISABLED

#define INTS_CAN_WAK_ISR_USED           INTS_CORE_ISR_DISABLED
#define INTS_CAN_WAK_INT_PRIORITY       INTS_LOW_PRIORITY_INTERRUPT
#define INTS_CAN_WAK_APPLICATION_HOOK   INTS_HOOK_DISABLED

// CORE NOTE: OTHER POSSIBLE INTERRUPTS TO BE ADDED LATER:
//
//      Parallel port,
//      EEPROM
//      Bus Collision
//---------------------------------------------------------------------------

#endif          // _CONFIG_H
