/*---------------------------------------------------------------------------\
| Project:          Demo LV2 PIC18F458 CAN node firmware
|
| File:             main.c
|
| Description:      Mainline routines.
|
| Global Routines:  Main()
|
| Local Routines:   None
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 06/01/2002 | Stolen from ADG code.
\---------------------------------------------------------------------------*/


//------------------------------------------------------------------------
//  #includes
//------------------------------------------------------------------------

#include ".\piccore\core.h"   // Includes for the project (includes other includes!)

#if CORE_VERSION != 4
#error HEY! The core version has changed! Make SURE your interfaces match!
#endif

#include "demo.h"   // Includes for the demo firmware program



//------------------------------------------------------------------------
//  Declarations of local routines
//------------------------------------------------------------------------

void Initialize (void);


//------------------------------------------------------------------------
//  Global Variables
//------------------------------------------------------------------------

 bit    timer2Tick;



/*---------------------------------------------------------------------------\
| Function:     main
|
| Purpose:      Demo the LV2 Core Firmware
|
| Discussion:   Sends out CAN messages at some frequency, and regurgitates
|               any received messages.
|
| Expects:      -
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 08/10/2002 | Modified to core version 4. How cool is that?
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void main (void)
{
    static CanMessage_t tempMsg;
    static CanMessage_t tempMsg2;

    // Initialize everything and its brother

    Initialize();

    // TEST TEST TEST

    tempMsg.header.all = (0x0200 << 5);
    tempMsg.header.bits.length = 1;
    tempMsg.data[0] = 0x00;

    SendCanMessage(&tempMsg);


    // Loop for ever...and......ev....er....and.......ev............er.

    while (TRUE)
    {
        if (timer2Tick)
        {
            ++tempMsg.data[0];
            SendCanMessage(&tempMsg);
            timer2Tick = 0;
        }

        if (canMessageAvailable)
        {
            GetCanMessage(&tempMsg2);
            SendCanMessage(&tempMsg2);
        }
    }

}




/*---------------------------------------------------------------------------\
| Function:     Initialize
|
| Purpose:      Initialize everything. Really.
|
| Discussion:   Did I mention initialization?
|
| Expects:      To be called after a RESET - eg, registers reset to known
|               values and and interrupts off.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void Initialize (void)
{

    /*
        EVENTUALLY: Check for a reset: If there hasn't been a reset,
        then something has gone horribly wrong. Do a SOFTWARE RESET
        here to put everything in a known state.
    */

    // Setting up the CAN bus peripheral: We're passing constants
    // to the initialization routine so that we can just set the
    // initialize initialization :) here.

    const CanHeader_t canMaskB0 = (0x07FF << 5);
    const CanHeader_t canMaskB1 = (0x07FF << 5);
    const CanHeader_t CanFilterB0F0 = (0x07FC <<5);
    const CanHeader_t CanFilterB0F1 = (0x0201 << 5);
    const CanHeader_t CanFilterB1F2 = (0x0000 << 5);
    const CanHeader_t CanFilterB1F3 = (0x0000 << 5);
    const CanHeader_t CanFilterB1F4 = (0x0000 << 5);
    const CanHeader_t CanFilterB1F5 = (0x0000 << 5);

    // Get the outputs stabilized FIRST.

    InitializePorts (  0b00000000, // latchA
                       0b00000000, // directionA (1 = in, 0 = out)
                       0b00000000, // latchB
                       0b00001100, // directionB (1 = in, 0 = out) (CAN!!)
                       0b00000000, // latchC
                       0b00000000, // directionC (1 = in, 0 = out)
                       0b00000000, // latchD
                       0b00000000, // directionD (1 = in, 0 = out)
                       0b00000000, // latchE
                       0b00000000, // directionE (1 = in, 0 = out)
                       (PORTS_PORTB_PULLUPS_OFF
                       | PORTS_INT0_RISING_EDGE
                       | PORTS_INT1_RISING_EDGE) );

    // Turn on serial transmitter and receiver. Turn on receiver interrupts
    // but leave the transmitter interrupts off (we'll turn them on when
    // there's actual data to send over the port). Make both interrupts
    // low priority, and se the bit rate to 9600bps.

    InitializeSerial (  SERIAL_TRANSMIT_ENABLE
                        | SERIAL_TRANSMIT_DISABLE_INTS
                        | SERIAL_RECEIVE_ENABLE
                        | SERIAL_RECEIVE_DISABLE_INTS
                        | SERIAL_BITRATE_9600 );

    // Turn on timer2, HP interrupts @ 1KHz

    InitializeTimer2 (  TIMER2_ENABLE
                        | TIMER2_ENABLE_INTS
                        | TIMER2_1KHZ );

    // Turn on CAN: See const variable declarations above
    // for the values. Note that as soon as we initialize the
    // the bus, we can start to receive messages, so be sure
    // that interrupts are enabled VERY SOON after this call
    // (No more than about a  message length, which at top
    // speed is about 1/22KHz = 45us which at 20MHz is only
    // 250 instructions.

    InitializeCan ( CAN_BITRATE_1MBPS,
                    canMaskB0,          // Mask RXB0
                    canMaskB1,          // Mask RXB1
                    CanFilterB0F0,      // Filter RXB0 F0
                    CanFilterB0F1,      // Filter RXB0 F1
                    CanFilterB1F2,      // Filter RXB1 F2
                    CanFilterB1F3,      // Filter RXB1 F3
                    CanFilterB1F4,      // Filter RXB1 F4
                    CanFilterB1F5);     // Filter RXB1 F5


    // ALL INITIALIZATION THAT MUST BE DONE PRIOR TO INTERRUPTS
    // BEING ENABLED MUST BE DONE PRIOR TO "InitializeInterrupts()".

    InitializeInterrupts();

    // INTERRUPTS ARE INITIALIZED; FEEL FREE TO INITIALIZE THINGS
    // WHICH REQUIRE INTERRUPTS TO BE RUNNING HERE.
}



