/*---------------------------------------------------------------------------\
| Project:  PIC18F458 CAN Node Core Firmware
|
| File:     handlers.c
|
| Descr.:   Application specific code which must be executed on an interrupt.
|           Essentially, these are "hooks" into the core's interrupt service
|           routines.
|
|           YOU SHOULD MODIFIY THIS FILE FOR YOUR APPLICATION, ALONG WITH
|           THE CONFIG.H FILE. FEEL FREE TO EVEN CUT AND COPY THESE FUNCTIONS
|           TO OTHER FILES.
|
|           HOWEVER, DO *NOT* CHANGE THE #IF, OR THE NAME AND TYPE OF THE
|           HANDLER FUNCTIONS.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 09/07/2002 | Created to store examples of handlers.
\---------------------------------------------------------------------------*/

#include ".\piccore\core.h"
#include "imu.h"

/*---------------------------------------------------------------------------\
| Function:		ADCApplicationHandler
|
| Purpose:		Set the temp_go flag when pic adc is done its conversion
|
| Discussion:	See purpose
|
| Expects:		The ADON=1, GODONE previously asserted.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 09/12/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void ADCApplicationHandler (void){
	temp_go = 1;
}


// I commented out the line below this to get rid of an
// linking error ASK ANDREW WHAT I DID.
 #if INTS_TIMER2_APPLICATION_HOOK == INTS_HOOK_ENABLED

/*---------------------------------------------------------------------------\
| Function:		Application_Timer2_Handler
|
| Purpose:		Set the correct bits in the Sample_ADC
|
| Discussion:	By compairing the global vars of sample rates against
|               Those of the local sample rate count.  Integer divide
|               by count not mod.
|
| Expects:		The four global sample rate variables to be set.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 09/07/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void Timer2ApplicationHandler(){

	static uint8_t AccX_Sample_Divisor_Counter = 1;
	static uint8_t AccY_Sample_Divisor_Counter = 1;
	static uint8_t AccZ_Sample_Divisor_Counter = 1;
	static uint8_t AccQ_Sample_Divisor_Counter = 1;
	static uint8_t GyroP_Sample_Divisor_Counter = 1;
	static uint8_t GyroY_Sample_Divisor_Counter = 1;
	static uint8_t GyroR_Sample_Divisor_Counter = 1;
	static uint8_t Temp_Sample_Divisor_Counter = 1;
	static uint8_t Pres_Sample_Divisor_Counter = 1;

//Start to set sensor flags
    //set flags fro accelorometers
	if(AccX_Sample_Divisor_Counter >= AccX_Sample_Divisor){
		AccX_Flag = TRUE;
		AccX_Sample_Divisor_Counter = 1;
	} else {
		AccX_Flag = FALSE;
	    ++AccX_Sample_Divisor_Counter;
	}

	if(AccY_Sample_Divisor_Counter >= AccY_Sample_Divisor){
		AccY_Flag = TRUE;
		AccY_Sample_Divisor_Counter = 1;
	} else {
		AccY_Flag = FALSE;
	    ++AccY_Sample_Divisor_Counter;
	}

	if(AccZ_Sample_Divisor_Counter >= AccZ_Sample_Divisor){
		AccZ_Flag = TRUE;
		AccZ_Sample_Divisor_Counter = 1;
	} else {
		AccZ_Flag = FALSE;
	    ++AccZ_Sample_Divisor_Counter;
	}

	if(AccQ_Sample_Divisor_Counter >= AccQ_Sample_Divisor){
		AccQ_Flag = TRUE;
		AccQ_Sample_Divisor_Counter = 1;
	} else {
		AccQ_Flag = FALSE;
	    ++AccQ_Sample_Divisor_Counter;
	}

    //set flags for gyroscope
	if(GyroP_Sample_Divisor_Counter >= GyroP_Sample_Divisor){
		GyroP_Sample_Divisor_Counter = 1;
		GyroP_Flag = TRUE;
	} else {
		GyroP_Flag = FALSE;
		++GyroP_Sample_Divisor_Counter;
	}

	if(GyroY_Sample_Divisor_Counter >= GyroY_Sample_Divisor){
		GyroY_Sample_Divisor_Counter = 1;
		GyroY_Flag = TRUE;
	} else {
		GyroY_Flag = FALSE;
		++GyroY_Sample_Divisor_Counter;
	}

	if(GyroR_Sample_Divisor_Counter >= GyroR_Sample_Divisor){
		GyroR_Sample_Divisor_Counter = 1;
		GyroR_Flag = TRUE;
	} else {
		GyroR_Flag = FALSE;
		++GyroR_Sample_Divisor_Counter;
	}

    //set pressure sensor flag
	if(Pres_Sample_Divisor_Counter >= Pres_Sample_Divisor){
        Pres_Flag = TRUE;
		Pres_Sample_Divisor_Counter = 1;
	} else {
		Pres_Flag = FALSE;
		++Pres_Sample_Divisor_Counter;
	}

    //set temperature sensor flag
	if(Temp_Sample_Divisor_Counter >= Temp_Sample_Divisor){
        Temp_Flag = TRUE;
		Temp_Sample_Divisor_Counter = 1;
	} else {
		Temp_Flag = FALSE;
		++Temp_Sample_Divisor_Counter;
	}
//end sensor flag setting


// Start taking first ADC sample the rest will be taken care of
// by Int0_Handler subroutine.  Because The temperature data is not
// converted with the Max197 it needs to be handeled with the
// 18f458's internal ADC.

    if(AccX_Flag){
		Max197_Cmd = 0b01000010;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		if(Temp_Flag){
			GODONE = 1;
		}
 		return;
	}
	if(AccY_Flag) {
		Max197_Cmd = 0b01000111;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		if(Temp_Flag){
			GODONE = 1;
		}
		return;
	}
	if(AccZ_Flag) {
		Max197_Cmd = 0b01000000;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		if(Temp_Flag){
			GODONE = 1;
		}
		return;
	}
	if(AccQ_Flag) {
		Max197_Cmd = 0b01000011;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		if(Temp_Flag){
			GODONE = 1;
		}
		return;
	}
	if(GyroP_Flag) {
		Max197_Cmd = 0b01000100;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		if(Temp_Flag){
			GODONE = 1;
		}
		return;
	}
	if(GyroY_Flag) {
		Max197_Cmd = 0b01000101;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		if(Temp_Flag){
			GODONE = 1;
		}
		return;
	}
	if(GyroR_Flag) {
		Max197_Cmd = 0b01000001;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		if(Temp_Flag){
			GODONE = 1;
		}
		return;
	}
	if(Pres_Flag) {
		Max197_Cmd = 0b01000110;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		if(Temp_Flag){
			GODONE = 1;
		}
		return;
	}
	if(Temp_Flag) {
		GODONE = 1;
	}

} // end Application_Timer2_Handler

#endif



/*---------------------------------------------------------------------------\
| Function:	Int0_Interupt
|
| Purpose:	Take in and process data from the MAX197, clear flags as they GODONE.
|
| Discussion:  It is very IMPORTANT that this makes its calls to the
|              MAX197 in the same order as the Application_Timer2_Handler
|              NOTE:That temperature AD conversion is handeled by the
|                   18f458's internal ADC.
|
| Expects:	Global flags for each sensor that needs to be sampled, at least
|           one of then needs to have been sampled before this function
|           would be called.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 09/07/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void Int0ApplicationHandler (){

 if(AccX_Flag){

	 TRISD = 0xff; //switch to input mode
	 RD_ = 0; //assert the read pin
	 AccX_ADC_Result_L = PORTD; //Read in the first low byte
	 HBEN = 1; //asser the High Byte Enable pin.
	 AccX_ADC_Result_H = PORTD;
	 HBEN = 0; //deassert High Byte Enable pin
	 RD_ = 1; // Deassert the read pin
	 TRISD = 0x00; //Deassert the
	 AccX_Flag = 0; //Clear this sensor flag
     if( ! (AccY_Flag | AccZ_Flag | AccQ_Flag) ){
		 acc_go =1;
     }

	 //look for and start next
	 if(AccY_Flag) {
	 		Max197_Cmd = 0b01000111;
	 		LATD = Max197_Cmd;
	 		WR_ = 0;
	 		WR_ = 1;
	 		return;
	 }
	 if(AccZ_Flag) {
	 		Max197_Cmd = 0b01000000;
	 		LATD = Max197_Cmd;
	 		WR_ = 0;
	 		WR_ = 1;
	 		return;
	 }
	 if(AccQ_Flag) {
	 		Max197_Cmd = 0b01000011;
	 		LATD = Max197_Cmd;
	 		WR_ = 0;
	 		WR_ = 1;
	 		return;
	 }
	 if(GyroP_Flag) {
	 		Max197_Cmd = 0b01000100;
	 		LATD = Max197_Cmd;
	 		WR_ = 0;
	 		WR_ = 1;
	 		return;
	 }
	 if(GyroY_Flag) {
	 		Max197_Cmd = 0b01000101;
	 		LATD = Max197_Cmd;
	 		WR_ = 0;
	 		WR_ = 1;
	 		return;
	 }
	 if(GyroR_Flag) {
	 		Max197_Cmd = 0b01000001;
	 		LATD = Max197_Cmd;
	 		WR_ = 0;
	 		WR_ = 1;
	 		return;
	 }
	 if(Pres_Flag) {
	 		Max197_Cmd = 0b01000110;
	 		LATD = Max197_Cmd;
	 		WR_ = 0;
	 		WR_ = 1;
	 		return;
	 }

 } // end check for AccX_Flag

 if(AccY_Flag){
	TRISD = 0xff; //switch to input mode
	RD_ = 0; //assert the read pin
	AccY_ADC_Result_L = PORTD; //Read in the first low byte
	HBEN = 1; //asser the High Byte Enable pin.
	AccY_ADC_Result_H = PORTD;
	HBEN = 0; //deassert High Byte Enable pin
	RD_ = 1; // Deassert the read pin
	TRISD = 0x00; //Deassert the
	AccY_Flag = 0; //Clear this sensor flag
	if( ! (AccZ_Flag | AccQ_Flag) ){
			 acc_go =1;
	}

	if(AccZ_Flag) {
		Max197_Cmd = 0b01000000;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		return;
	}
	if(AccQ_Flag) {
		Max197_Cmd = 0b01000011;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		return;
	}
	if(GyroP_Flag) {
		Max197_Cmd = 0b01000100;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		return;
	}
	if(GyroY_Flag) {
		Max197_Cmd = 0b01000101;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		return;
	}
	if(GyroR_Flag) {
		Max197_Cmd = 0b01000001;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		return;
	}
	if(Pres_Flag) {
		Max197_Cmd = 0b01000110;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		return;
	}
 }

 if(AccZ_Flag){
	TRISD = 0xff; //switch to input mode
	RD_ = 0; //assert the read pin
	AccZ_ADC_Result_L = PORTD; //Read in the first low byte
	HBEN = 1; //asser the High Byte Enable pin.
	AccZ_ADC_Result_H = PORTD; // special read thingy
	HBEN = 0; //deassert High Byte Enable pin
	RD_ = 1; // Deassert the read pin
	TRISD = 0x00; //Deassert the
	AccZ_Flag = 0; //Clear this sensor flag
	if( ! (AccQ_Flag) ){
			 acc_go =1;
	}

	// check to see if any other sensors need to be sampeled
	if(AccQ_Flag) {
		Max197_Cmd = 0b01000011;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		return;
	}
	if(GyroP_Flag) {
		Max197_Cmd = 0b01000100;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		return;
	}
	if(GyroY_Flag) {
		Max197_Cmd = 0b01000101;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		return;
	}
	if(GyroR_Flag) {
		Max197_Cmd = 0b01000001;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		return;
	}
	if(Pres_Flag) {
		Max197_Cmd = 0b01000110;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		return;
	}
 }
 if(AccQ_Flag){
	TRISD = 0xff; //switch to input mode
	RD_ = 0; //assert the read pin
	AccQ_ADC_Result_L = PORTD; //Read in the first low byte
	HBEN = 1; //asser the High Byte Enable pin.
	AccQ_ADC_Result_H = PORTD; // special read thingy
	HBEN = 0; //deassert High Byte Enable pin
	RD_ = 1; // Deassert the read pin
	TRISD = 0x00; //Deassert the
	AccQ_Flag = 0; //Clear this sensor flag
    acc_go = 1;

	// check to see if any other sensors need to be sampeled
	if(GyroP_Flag) {
		Max197_Cmd = 0b01000100;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		return;
	}
	if(GyroY_Flag) {
		Max197_Cmd = 0b01000101;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		return;
	}
	if(GyroR_Flag) {
		Max197_Cmd = 0b01000001;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		return;
	}
	if(Pres_Flag) {
		Max197_Cmd = 0b01000110;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		return;
	}
 }

 if(GyroP_Flag){

	TRISD = 0xff; //switch to input mode
	RD_ = 0; //assert the read pin
	GyroP_ADC_Result_L = PORTD; //Read in the first low byte
	HBEN = 1; //asser the High Byte Enable pin.
	GyroP_ADC_Result_H = PORTD; // special read thingy
	HBEN = 0; //deassert High Byte Enable pin
	RD_ = 1; // Deassert the read pin
	TRISD = 0x00; //Deassert the
	GyroP_Flag = 0; //Clear this sensor flag
    if(! (GyroY_Flag | GyroR_Flag)){
		gyro_go = 1;
	}
	// check to see if any other sensors need to be sampeled
	if(GyroY_Flag) {
		Max197_Cmd = 0b01000101;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		return;
	} else if(GyroR_Flag) {
		Max197_Cmd = 0b01000001;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		return;
	} else if(Pres_Flag) {
		Max197_Cmd = 0b01000110;
		LATD = Max197_Cmd;
		WR_ = 0;
		WR_ = 1;
		return;
	} else {
		return;
	}
 }

 if(GyroY_Flag){
	 TRISD = 0xff; //switch to input mode
	 RD_ = 0; //assert the read pin
	 GyroY_ADC_Result_L = PORTD; //Read in the first low byte
	 HBEN = 1; //asser the High Byte Enable pin.
	 GyroY_ADC_Result_H = PORTD;
	 HBEN = 0; //deassert High Byte Enable pin
	 RD_ = 1; // Deassert the read pin
	 TRISD = 0x00; //Deassert the
	 GyroY_Flag = 0; //Clear this sensor flag
	 if(! (GyroR_Flag)){
		gyro_go = 1;
     }
	 // check to see if any other sensors need to be sampeled
	 if(GyroR_Flag) {
	 	Max197_Cmd = 0b01000001;
	 	LATD = Max197_Cmd;
	 	WR_ = 0;
	 	WR_ = 1;
	 	return;
	 } else if(Pres_Flag) {
	 	Max197_Cmd = 0b01000110;
	 	LATD = Max197_Cmd;
	 	WR_ = 0;
	 	WR_ = 1;
	 	return;
	 } else {
	 	return;
	 }
 }

 if(GyroR_Flag){
	 TRISD = 0xff; //switch to input mode
	 RD_ = 0; //assert the read pin
	 GyroR_ADC_Result_L = PORTD; //Read in the first low byte
	 HBEN = 1; //asser the High Byte Enable pin.
	 GyroR_ADC_Result_H = PORTD; // special read thingy
	 HBEN = 0; //deassert High Byte Enable pin
	 RD_ = 1; // Deassert the read pin
	 TRISD = 0x00; //Deassert the
	 GyroR_Flag = 0; //Clear this sensor flag
     gyro_go = 1;
	 // check to see if any other sensors need to be sampeled
	 if(Pres_Flag) {
	 	 	Max197_Cmd = 0b01000110;
	 	 	LATD = Max197_Cmd;
	 	 	WR_ = 0;
	 	 	WR_ = 1;
	 	 	return;
	 	 } else {
	 	 	return;
	 }
 }

 if(Pres_Flag){
	 TRISD = 0xff; //switch to input mode
	 RD_ = 0; //assert the read pin
	 Pres_ADC_Result_L = PORTD; //Read in the first low byte
	 HBEN = 1; //asser the High Byte Enable pin.
	 Pres_ADC_Result_H = PORTD; // special read thingy
	 HBEN = 0; //deassert High Byte Enable pin
	 RD_ = 1; // Deassert the read pin
	 TRISD = 0x00; //Deassert the
	 Pres_Flag = 0; //Clear this sensor flag
	 pres_go = 1;
	 return;
 }


} // end Int0_Interupt
