/*---------------------------------------------------------------------------\
| Project:			Generic LV2 PIC18F458 CAN node firmware
|
| File:				main.c
|
| Description:		Mainline routines.
|
| Global Routines:	Main()
|
| Local Routines:	None
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 06/01/2002 | Stolen from ADG code.
\---------------------------------------------------------------------------*/



//------------------------------------------------------------------------
//  #includes
//------------------------------------------------------------------------

#include ".\piccore\core.h"   // Includes for the project (includes other includes!)

#if CORE_VERSION != 4
#error HEY! The core version has changed! Make SURE your interfaces match!
#endif

#include "imu.h"   // Includes for the imu firmware program

//------------------------------------------------------------------------
// Global variable declarations
//------------------------------------------------------------------------
//These globals are used to define the sample rates for IMU and Temp and Pressure
uint8_t AccX_Sample_Divisor = 1; //Default rate of 2.5kHz (w/timer2@2.5kHz)
uint8_t AccY_Sample_Divisor = 1; //Default rate of 2.5kHz (w/timer2@2.5kHz)
uint8_t AccZ_Sample_Divisor = 1; //Default rate of 2.5kHz (w/timer2@2.5kHz)
uint8_t AccQ_Sample_Divisor = 1; //Default rate of 2.5kHz (w/timer2@2.5kHz)
uint8_t GyroP_Sample_Divisor = 3;//Default rate of 833Hz (w/timer2@2.5kHz)
uint8_t GyroY_Sample_Divisor = 3;//Default rate of 833Hz (w/timer2@2.5kHz)
uint8_t GyroR_Sample_Divisor = 3;//Default rate of 833Hz (w/timer2@2.5kHz)
uint8_t Pres_Sample_Divisor = 25;//Default rate of 100Hz (w/timer2@2.5kHz)
uint8_t Temp_Sample_Divisor = 250;//Default rate of 10Hz (w/timer2@2.5kHz)

// if a sample needs to be taken its associated flag will be set to TRUE
bit AccX_Flag;
bit AccY_Flag;
bit AccZ_Flag;
bit AccQ_Flag;
bit GyroP_Flag;
bit GyroY_Flag;
bit GyroR_Flag;
bit Pres_Flag;
bit Temp_Flag;

// holds commands for Max197 AD chip, see data sheet for more details
uint8_t Max197_Cmd;

// hold results from the ADC before sending in the CAN message
uint8_t AccX_ADC_Result_H;
uint8_t AccY_ADC_Result_H;
uint8_t AccZ_ADC_Result_H;
uint8_t AccQ_ADC_Result_H;
uint8_t GyroP_ADC_Result_H;
uint8_t GyroY_ADC_Result_H;
uint8_t GyroR_ADC_Result_H;
uint8_t Pres_ADC_Result_H;

uint8_t AccX_ADC_Result_L;
uint8_t AccY_ADC_Result_L;
uint8_t AccZ_ADC_Result_L;
uint8_t AccQ_ADC_Result_L;
uint8_t GyroP_ADC_Result_L;
uint8_t GyroY_ADC_Result_L;
uint8_t GyroR_ADC_Result_L;

uint8_t Pres_ADC_Result_L;
uint8_t Pres_ADC_Result_H;

// flags for when a can message is ready to be sent
bit acc_go;
bit gyro_go;
bit pres_go;
bit temp_go;

/*---------------------------------------------------------------------------\
| Function:		main
|
| Purpose:		Run all Mainline code
|
| Discussion:	Doesn't do much, huh?
|
| Expects:		Uh let's see... nothing.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void main (void)
{
	static CanMessage_t	tempMsg;
	// static CanMessage_t	tempMsg2;

	// Initialize everything and its brother

	Initialize();


	// Loop for ever...and......ev....er....and.......ev............er.

	while (TRUE)
	{
		if(acc_go){
			tempMsg.header.all = IMUID_ACC;
			tempMsg.data[0] = AccX_ADC_Result_H;
			tempMsg.data[1] = AccX_ADC_Result_L;
			tempMsg.data[2] = AccY_ADC_Result_H;
			tempMsg.data[3] = AccY_ADC_Result_L;
			tempMsg.data[4] = AccZ_ADC_Result_H;
			tempMsg.data[5] = AccZ_ADC_Result_L;
			tempMsg.data[6] = AccQ_ADC_Result_H;
			tempMsg.data[7] = AccQ_ADC_Result_L;
			acc_go = 0;
            SendCanMessage(&tempMsg);
		}
		if(gyro_go){
			tempMsg.header.all = IMUID_GYRO;
			tempMsg.data[0] = GyroP_ADC_Result_H;
			tempMsg.data[1] = GyroP_ADC_Result_L;
			tempMsg.data[2] = GyroY_ADC_Result_H;
			tempMsg.data[3] = GyroY_ADC_Result_L;
			tempMsg.data[4] = GyroR_ADC_Result_H;
			tempMsg.data[5] = GyroR_ADC_Result_L;
			SendCanMessage(&tempMsg);
			gyro_go = 0;
		}
		if(pres_go){
			tempMsg.header.all = PresID_WORD;
			tempMsg.data[0] = Pres_ADC_Result_L;
			tempMsg.data[1] = Pres_ADC_Result_H;
			SendCanMessage(&tempMsg);
			pres_go = 0;
		}
	    if(temp_go){
			tempMsg.header.all = TempID_WORD;
			tempMsg.data[0] = ADRESL;
			tempMsg.data[1] = ADRESH; //expects ADFM=1 (pg234 458 datasheet)
			SendCanMessage(&tempMsg);
			temp_go = 0;
		}
		if(canMessageAvailable){
			GetCanMessage(&tempMsg);
			Parse_Can_Message(&tempMsg);
			//reset to parse flag
			canMessageAvailable = 0;
		}
	}

}







/*---------------------------------------------------------------------------\
| Function:     Initialize
|
| Purpose:      Initialize everything. Really.
|
| Discussion:   Did I mention initialization?
|
| Expects:      To be called after a RESET - eg, registers reset to known
|               values and and interrupts off.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void Initialize (void)
{

    /*
        EVENTUALLY: Check for a reset: If there hasn't been a reset,
        then something has gone horribly wrong. Do a SOFTWARE RESET
        here to put everything in a known state.
    */

    // Setting up the CAN bus peripheral: We're passing constants
    // to the initialization routine so that we can just set the
    // initialize initialization :) here.

    const CanHeader_t canMaskB0 = (0x07FF << 5);
    const CanHeader_t canMaskB1 = (0x07FF << 5);
    const CanHeader_t CanFilterB0F0 = (0x07FC <<5);
    const CanHeader_t CanFilterB0F1 = (0x0201 << 5);
    const CanHeader_t CanFilterB1F2 = (0x0000 << 5);
    const CanHeader_t CanFilterB1F3 = (0x0000 << 5);
    const CanHeader_t CanFilterB1F4 = (0x0000 << 5);
    const CanHeader_t CanFilterB1F5 = (0x0000 << 5);

    // Initiaize the ADC for use by the Temp sensor.
	// Start by selecting the ADC channel on the PIC
		CHS0 = 0;
		CHS1 = 0;
		CHS2 = 0;
	// Next turn on the ADC
		ADON = 1;
		GODONE = 0;
	// Make the 6 most significant bits of ADRESH are 0's
	    ADFM = 1;
	// Set ADC conversion clock select bits
	// this config insures Fosc/32
	    ADCS2 = 0;
	    ADCS1 = 0;
	    ADCS0 = 0;

    // Get the outputs stabilized FIRST.

    InitializePorts (  0b00000000, // latchA
                       0b00000000, // directionA (1 = in, 0 = out)
                       0b00000000, // latchB
                       0b00001100, // directionB (1 = in, 0 = out) (CAN!!)
                       0b00000000, // latchC
                       0b00000000, // directionC (1 = in, 0 = out)
                       0b00000000, // latchD
                       0b00000000, // directionD (1 = in, 0 = out)
                       0b00000000, // latchE
                       0b00000000, // directionE (1 = in, 0 = out)
                       (PORTS_PORTB_PULLUPS_OFF
                       | PORTS_INT0_RISING_EDGE
                       | PORTS_INT1_RISING_EDGE) );

    // Turn on serial transmitter and receiver. Turn on receiver interrupts
    // but leave the transmitter interrupts off (we'll turn them on when
    // there's actual data to send over the port). Make both interrupts
    // low priority, and se the bit rate to 9600bps.

    InitializeSerial (  SERIAL_TRANSMIT_DISABLE
                        | SERIAL_TRANSMIT_DISABLE_INTS
                        | SERIAL_RECEIVE_DISABLE
                        | SERIAL_RECEIVE_DISABLE_INTS
                        | SERIAL_BITRATE_9600 );

    // Turn on timer2, HP interrupts @ 1KHz
/*
    InitializeTimer2 (  TIMER2_ENABLE
                        | TIMER2_ENABLE_INTS
                        | TIMER2_1KHZ );
     Teh above code gave this error when compieling after switching to
     the new timer2.c and timer2.h 9/13/02
     Error[000] C:\FIRMWARE\IMU.C 254 : undefined identifier: TIMER2_ENABLE
     switchted to the code below.
                        */
InitializeTimer2 (TIMER2_2_5KHZ | TIMER2_ENABLE_INTS);

    // Turn on CAN: See const variable declarations above
    // for the values. Note that as soon as we initialize the
    // the bus, we can start to receive messages, so be sure
    // that interrupts are enabled VERY SOON after this call
    // (No more than about a  message length, which at top
    // speed is about 1/22KHz = 45us which at 20MHz is only
    // 250 instructions.

    InitializeCan ( CAN_BITRATE_1MBPS,
                    canMaskB0,          // Mask RXB0
                    canMaskB1,          // Mask RXB1
                    CanFilterB0F0,      // Filter RXB0 F0
                    CanFilterB0F1,      // Filter RXB0 F1
                    CanFilterB1F2,      // Filter RXB1 F2
                    CanFilterB1F3,      // Filter RXB1 F3
                    CanFilterB1F4,      // Filter RXB1 F4
                    CanFilterB1F5);     // Filter RXB1 F5


    // ALL INITIALIZATION THAT MUST BE DONE PRIOR TO INTERRUPTS
    // BEING ENABLED MUST BE DONE PRIOR TO "InitializeInterrupts()".

    InitializeInterrupts();

    // INTERRUPTS ARE INITIALIZED; FEEL FREE TO INITIALIZE THINGS
    // WHICH REQUIRE INTERRUPTS TO BE RUNNING HERE.

}



/*---------------------------------------------------------------------------\
| Function:     Parse_Can_Message
|
| Purpose:      Interpet can message and act on it.
|
| Discussion:   see purpose
|
| Expects:      a can message with directions in it
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void Parse_Can_Message(CanMessage_t* receivedMsg){


    switch(receivedMsg->header.all & (0x7FF<<5) ){
		// IMU base sample rate (Timer2 rate)
		case (0x041 << 5):
		  switch(receivedMsg->data[0]){
			  case 0:
			    InitializeTimer2(TIMER2_OFF);
			  break;
			  case 1:
			    InitializeTimer2(TIMER2_250HZ);
			  break;
			  case 2:
			    InitializeTimer2(TIMER2_500HZ);
			  break;
			  case 3:
			    InitializeTimer2(TIMER2_1KHZ);
			  break;
			  case 4:
			    InitializeTimer2(TIMER2_2KHZ);
			  break;
			  case 5:
			    InitializeTimer2(TIMER2_2_5KHZ);
			  break;
			  case 6:
			    InitializeTimer2(TIMER2_5KHZ);
			  break;
			  case 7:
			    InitializeTimer2(TIMER2_10KHZ);
			  break;
			  default:
			    ThrowWarning(WARN_BASE_RATE_TIMER2_OUT_OF_RANGE);
			  break;
		  }
		break;
        // IMU ID integer divide (Sample Rate Change)
		case (0x042 << 5):
			if(receivedMsg->data[0]){
				//change sample rate for acc
				AccX_Sample_Divisor = receivedMsg->data[1];
				AccY_Sample_Divisor = receivedMsg->data[1];
				AccZ_Sample_Divisor = receivedMsg->data[1];
				AccQ_Sample_Divisor = receivedMsg->data[1];
			} else {
				// change sample rete for gyro
				GyroP_Sample_Divisor = receivedMsg->data[1];
				GyroY_Sample_Divisor = receivedMsg->data[1];
				GyroR_Sample_Divisor = receivedMsg->data[1];
			}
		break;
		// Pressure ID Sample Rate Change
		case (0x071 << 5):
			Pres_Sample_Divisor = receivedMsg->data[0];
		break;
		// Temp ID Sample Rate Change
		case (0x141 << 5):
			Temp_Sample_Divisor = receivedMsg->data[0];
		break;
		default:
			ThrowWarning(WARN_INTEGER_DIVISOR_SENSOR_ID_BAD);
		break;
	} // end ID switch


} // end Parse_Can_Message

