/*---------------------------------------------------------------------------\
| Project:  imu LV2 PIC18F458 CAN node firmware
|
| File:     imu.h
|
| Descr.:   Demo.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 06/01/2002 | Created.
\---------------------------------------------------------------------------*/

#ifndef _IMU_H     // Don't include more than once (last line = #endif)
#define _IMU_H

//------------------------------------------------------------------------
// Typedefs
//------------------------------------------------------------------------


enum WARNING_MESSAGES
{
	WARN_BASE_RATE_TIMER2_OUT_OF_RANGE,
	WARN_INTEGER_DIVISOR_SENSOR_ID_BAD
};


//------------------------------------------------------------------------
// #defines
//------------------------------------------------------------------------
// #define the pins of the 18F458
#define IMUTEMP_ RA0 //PIN# 19
#define CS_ RE0      //PIN 25
#define HBEN RE2     //PIN 27
#define RD_ RC0      //PIN 32
#define WR_ RC1      //PIN 35
#define TEST RC2     //PIN 36
#define INT_ RB0     //PIN 8
#define D0 RD0       //PIN 38
#define D1 RD1       //PIN 39
#define D2 RD2       //PIN 40
#define D3 RD3       //PIN 41
#define D4 RD4       //PIN 2
#define D5 RD5       //PIN 3
#define D6 RD6       //PIN 4
#define D7 RD7       //PIN 5
#define CANRX RB3    //PIN 11
#define CANTX RB2    //PIN 10

//defines for Can message headers
#define IMUID_ACC   0b0001001000001000 // send imu acc data
#define IMUID_GYRO  0b0001001010000110 // send imu gyro data
#define PresID_WORD 0b0001110010000010 // send pressure data
#define TempID_WORD 0b0101000010000010 // send temp data


//------------------------------------------------------------------------
// Function declarations
//------------------------------------------------------------------------
void Initialize (void);
void Parse_Can_Message(CanMessage_t* receivedMsg);


// None


//------------------------------------------------------------------------
// Global variable declarations
//------------------------------------------------------------------------
//These globals are used to define the sample rates for IMU and Temp and Pressure
extern uint8_t AccX_Sample_Divisor; //Default rate of 2.5kHz (w/timer2@2.5kHz)
extern uint8_t AccY_Sample_Divisor; //Default rate of 2.5kHz (w/timer2@2.5kHz)
extern uint8_t AccZ_Sample_Divisor; //Default rate of 2.5kHz (w/timer2@2.5kHz)
extern uint8_t AccQ_Sample_Divisor; //Default rate of 2.5kHz (w/timer2@2.5kHz)
extern uint8_t GyroP_Sample_Divisor;//Default rate of 833Hz (w/timer2@2.5kHz)
extern uint8_t GyroY_Sample_Divisor;//Default rate of 833Hz (w/timer2@2.5kHz)
extern uint8_t GyroR_Sample_Divisor;//Default rate of 833Hz (w/timer2@2.5kHz)
extern uint8_t Pres_Sample_Divisor;//Default rate of 100Hz (w/timer2@2.5kHz)
extern uint8_t Temp_Sample_Divisor;//Default rate of 10Hz (w/timer2@2.5kHz)

// if a sample needs to be taken its associated flag will be set to TRUE
extern bit AccX_Flag;
extern bit AccY_Flag;
extern bit AccZ_Flag;
extern bit AccQ_Flag;
extern bit GyroP_Flag;
extern bit GyroY_Flag;
extern bit GyroR_Flag;
extern bit Pres_Flag;
extern bit Temp_Flag;

// holds commands for Max197 AD chip, see data sheet for more details
extern uint8_t Max197_Cmd;

// hold results from the ADC before sending in the CAN message
extern uint8_t AccX_ADC_Result_H;
extern uint8_t AccY_ADC_Result_H;
extern uint8_t AccZ_ADC_Result_H;
extern uint8_t AccQ_ADC_Result_H;
extern uint8_t GyroP_ADC_Result_H;
extern uint8_t GyroY_ADC_Result_H;
extern uint8_t GyroR_ADC_Result_H;
extern uint8_t Pres_ADC_Result_H;

extern uint8_t AccX_ADC_Result_L;
extern uint8_t AccY_ADC_Result_L;
extern uint8_t AccZ_ADC_Result_L;
extern uint8_t AccQ_ADC_Result_L;
extern uint8_t GyroP_ADC_Result_L;
extern uint8_t GyroY_ADC_Result_L;
extern uint8_t GyroR_ADC_Result_L;

extern uint8_t Pres_ADC_Result_L;
extern uint8_t Pres_ADC_Result_H;

// flags for when a can message is ready to be sent
extern bit acc_go;
extern bit gyro_go;
extern bit pres_go;
extern bit temp_go;



//------------------------------------------------------------------------
// Module-wide variable declarations
//------------------------------------------------------------------------

// None


#endif              // _IMU_H


